/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

package hypercast;

/**
 * StatsException is used to throw errors associated with the I_Stats interface.
 * 
 * @author HyperCast Team
 * @author Neelima Putrevu
 * @author Michael Nahas
 * @version 2.0, Jan. 10, 2001
 */

public class HyperCastStatsException extends HyperCastException
{
    /** value for <code>type</code>. */
    public static final int StatisticNotFound = 1;

    /** value for <code>type</code>.*/
    public static final int WriteToAReadOnlyValue = 2;

    /** value for <code>type</code>.  Indicates that the child object's schema contained errors. */
    public static final int ChildSchemaViolatedFormatSpec = 3; 

    /** value for <code>type</code>. E.g. expected an integer and was passed letters.*/
    public static final int InappropriateType = 4;

    /** value for <code>type</code> E.g. was negative when expected a positive integer. */
    public static final int InappropriateValue = 5;

    /** The value format of a setValue command is incorrect. */
    public static final int IncorrectSetValue = 6;


    /**
     * If available, the name of the parameter that caused the
     * exception.
     */
    String name;

    /**
     * One of the type values defined by this class.
     */
    int type;
    
    /**
     * Create an exception with certain name and type.
     */
    public HyperCastStatsException (final String name, final int type)
    {
        super();
        this.name = name;
        this.type = type;
    }

    /**
     * @return an appropriate error message for the exception.
     */
    public String getMessage()
    {
        switch (type)
        {
            case StatisticNotFound:
                return "There is no statistic with the name \"" + name + "\"";
            case WriteToAReadOnlyValue:
                return name + " is a read-only value.";
            case ChildSchemaViolatedFormatSpec:
                return "Internal Schema violation";
            case InappropriateType:
                return "The value assigned to " + name + " was of an inappropriate type.";
            case InappropriateValue:
                return "The value assigned to " + name + " was inappropriate for some reason (e.g. out of range, file doesn't exist, etc.).";
            case IncorrectSetValue:
                return "The format of value assigned to " + name + " is incorrect.";
            default:
                return "Internal error: This StatsException object has an invalid type!!!";
        }
    }
}
