/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

package hypercast;

/**
 * This interface is implemented by a UDP/TCP unicast adapter or a UDP multicast adapter.
 * The method is called whenever there is a message arriving the adapter.
 * @author Dongwen Wang
 * @author Michael Nahas
 * @version	2.0, Feb 23, 2001
 * 
 * The parameter, timer type, of the timerExpired method is changed to a general "Object".
 *
 * @author HyperCast Team
 * @author Jianping Wang
 * @version     3.0, Dec. 05, 2003
 */
public interface I_AdapterCallback
{
    /**
     * Handles the incoming message (unicast, server, or multicast). 
     * @throws IllegalArgumentException if I_Message is of incorrect type.
     * (An incorrect type being one different than that returned by restoreMessage(). )
     */
    public void messageArrivedFromAdapter (I_Message msg);

    /**
     * Handles the arrival of a timer 
     * @throws IllegalArgumentException if timerID is unexpected.
     */ 
    public void timerExpired (Object timerID);
	
    /**
     * Creates a message from bytes in a buffer.
     * validBytesStart[0] is the start of the segment of unread bytes in receiveBuffer.
     * validBytesEnd is one beyond the end of the segment of unread bytes in receiveBuffer.
     * If a message is contained in the buffer, the function increases validBytesStart[0] to
     * point to the new beginning of unread bytes in the buffer.  If no message is contained
     * in the buffer, null is returned.  If the buffer contains corrupt or bad messages,
     * an IllegalArgumentException should be thrown.
     * @param	receiveBuffer		the byte array buffer of a incoming stream
     * @param	validBytesStart	the start index to create a message from
     * @param	validBytesEnd	the total length of bytes in this buffer
     * @return null if buffer does not contain a complete message.
     * @throws IllegalArgumentException
     */ 
    public I_Message restoreMessage (byte[] receiveBuffer, int[] validBytesStart, int validBytesEnd);
}

