/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

package hypercast;

/**
 * This interface defines functions that an unicast adapter should
 * implement.  An adapter provides timers and a network interface.  It
 * can send and receive messages.  (Messages received are passed to an
 * object implementing I_AdapterCallback.)  The adapter keeps its own
 * clock for the timers.  It is important that any objects implementing
 * I_AdapterCallback uses this clock, so that their
 * code will work with a simulated network.  It is implemented by the
 * class UDP_UnicastAdapter.
 * <p>
 *
 * @author HyperCast Team
 * @author Dongwen Wang
 * @version 2.0, Feb. 23, 2001
 * @author Jianping Wang
 * @version     3.0, Dec. 05, 2003
 */

public interface I_UnicastAdapter extends I_Stats {
    /** Starts all the threads except timer thread in the UnicastAdapter. 
         * @throws IllegalStateException if the adapter is already running.
         * @throws IllegalStateException if setCallback has not been called.
         */
	public void Start();

    /** Starts timer thread in the UnicastAdapter.
	*/ 
	public void startTimer();

   /** Stops all the threads in the UnicastAdapter. 
         * When stop returns, the adapter is stopped.  Any further calls    
         * (e.g. send, setTimer, clearTimer, etc.) will call an exception.
         * Stop() may take a while to return.  During that time, Callback
         * functions may be called.  Stop() will only return when all threads
         * have returned from the Callback functions and are guaranteed 
         * never to enter a Callback function.
         * @throws IllegalStateException if the adapter is already stopped.
         */
	public void Stop();
	
    /** Stops timer thread in the UnicastAdapter.
	*/ 
	public void stopTimer();

	
	/** Sends an I_Message to a specified destination from this Adapter. 
         * @throws IllegalStateException if adapter is not running.
         */
	public void sendUnicastMessage(I_NetworkAddress dst, I_Message msg);

	/** Creates an instance of an I_PhysicalAddress from a byte array. 
         * @throws IllegalArgumentException if byte[] is corrupted 
         * @throws ArrayIndexOutOfBoundsException if byte[] is too small.
         */
	public I_PhysicalAddress createPhysicalAddress(byte[] byteaddr, int offset);
	
	/** 
	 * Creates IP_Address object from a string.
	 * @param	addrString  an string representing a physical address.
	 */
	public I_PhysicalAddress createPhysicalAddress(String addrString);

	/** Create a physical address.
     */
	public I_PhysicalAddress createPhysicalAddress();

	/** Returns the size of physical address, when carried in a packet,
	 *  of an I_UnicastAdapter object. 
	 */
	public int getPhysicalAddressSize();

	/** Creates an instance of an I_UnderlayAddress from a byte array. 
		 * @throws IllegalArgumentException if byte[] is corrupted 
		 * @throws ArrayIndexOutOfBoundsException if byte[] is too small.
		 */
	public I_UnderlayAddress createUnderlayAddress(byte[] byteaddr, int offset);
	
	/** 
	 * Creates I_UnderlayAddress object from a string.
	 * @param	addrString  an string representing a underlay address.
	 */
	public I_UnderlayAddress createUnderlayAddress(String addrString);

	/** Create a physical address.
	 */
	public I_UnderlayAddress createUnderlayAddress();

	/** Returns the size of physical address, when carried in a packet,
	 *  of an I_UnicastAdapter object. 
	 */
	public int getUnderlayAddressSize();


	/** Reset the adapter with a Object;
	 * @param socketObj	object passed to do reset.
	 */
	public boolean Reset(Object socketObj);

	/** Get information of this adapter.
	*/ 
	public Object getInfo();



	/** Sets a Callback object to this Adapter.  This must be done
         * only when the Adapter is stopped, and must be performed
         * before the Adapter is started.
         *
         * @throws IllegalStateException if called after adapter has been started.  */
	public void setCallback (I_AdapterCallback node);
	
	/** Sets a timer.  If timer with that id has already be set, this resets it. 
         * @throws IllegalStateException if adapter is not running.
         */
    public void setTimer(Object timer_id, long delay_ms);

    /** Clears a timer. 
         * @throws IllegalStateException if adapter is not running.
         */
    public void clearTimer(Object timer_id);

    /** Returns expect delay until the timer goes off. 
         * @throws IllegalStateException if adapter is not running.
         */
    public long getTimer(Object timer_id);
	
    /** USE THIS - DO NOT USE <code>System.currentTimeMillis()</code>!
        * This returns the time according to the adapter.  It is
        * important that all code use this function and not the system
        * clock so that the code will work with a simulator (which may
        * be paused and restarted).  */
    public long getCurrentTime();

}


