/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

package hypercast;

import hypercast.events.NOTIFICATION_EVENT;
import hypercast.util.XmlUtil;
import hypercast.MessageStoreFSM_E2EACK;

import java.util.*;
/**
 * This class is a finite state machine which handles hop to hop ack.
 * 
 * @author HyperCast Team
 * @author Guimin Zhang
 * @version 2.0, May. 23, 2001 */

public class MessageStoreFSM_H2HACK extends I_MessageStoreFSM
{
	/** debug flag */
	boolean debug = false;
	
		/** OL_Socket object*/
	private OL_Socket socket;
		/** MessageStore*/
	private MessageStore MStore;
		/** logical address*/
	private I_LogicalAddress mylogicaladdress;
		/** root logical address*/
	private I_LogicalAddress root=null;
		/** logical address of the source of message*/
	private I_LogicalAddress sourceaddress;
	/** logical address of the previous hop of message*/
	private I_LogicalAddress previousddress;

	/** delivery mode*/
	private byte deliveryMode=OL_Message.DELIVERY_MODE_MULTICAST;
		/**ADF_Payload for H2H ACK*/
	public static final byte PAYLOAD					= -1;
		/**ADF_Retransmit payload for H2H ACK*/
	public static final byte PAYLOAD_RETRAN				= -2;
		/**ADF_Control Types for H2H ACK*/
	public static final byte H2H_NACK					= 1;
	public static final byte H2H_ACK					= 2;
	public static final byte H2H_REQUEST				= 3;
	public static final byte H2H_RESET					= 4;
	
	

		/** the message id */
	private byte[] messageid = null;	

	/** The message handled by this FSM */
	private OL_Message message = null;

	
		/** ACK Received*/
	private Vector ACKReceived;
		/** ACK Expected*/
	private Vector ACKExpected;
		/** ACK sent*/
	private boolean  ACKSent;
		/** The state information of the message */
	private byte MyState;
	
		/** the length of Children nodes*/
	private int length;
	
		/** The state of the message,Init */
	public static final byte Init = 0;
		/** The state of the message,Don't have payload */
	public static final byte NoPayLoad = 1;
		/** The state of the message,Wait for ACK*/
	public static final byte WaitforACK = 2;
		/** The state of the message,Done */
	public static final byte Done = 3;
		/** The state of the message,Reset */
	public static final byte Reset = 4;

	/** the ACK timeout id */
	private static final int ACK_TIMER_INDEX = 0;
	/** the max ACK timeout id */	
	private static final int MAXACK_TIMER_INDEX = 1;
	/**the NACK timeout id */
	private static final int NACK_TIMER_INDEX = 2;
	/**the max NACK timeout id */
	private static final int MAXNACK_TIMER_INDEX = 3;
	/**the RESET timeout id */
	private static final int RESET_TIMER_INDEX = 4;
	/**the max RESET timeout id */
	private static final int MAXRESET_TIMER_INDEX = 5;
	/**the delete timeout id */
	private static final int DELETE_TIMER_INDEX = 6;
	/**the final delete timeout id */
	private static final int FINALDELETE_TIMER_INDEX = 7;
	/**the routing change timeout id */
	private static final int ROUTINGCHANGE_TIMER_INDEX = 8;
		
	/** The timeout const,ACK(in millisecond) */
	private static long timeout_ACK = -1;
		/** The timeout const,max ACK(in millisecond) */
	private static long max_ACK;
		/** The timeout const,ACK(in millisecond) */
	private static long timeout_NACK;
		/** The timeout const,max NACK(in millisecond) */
	private static long max_NACK;	
		/** The timeout const,RESET(in millisecond) */
	private static long timeout_RESET;
		/** The timeout const,max RESET(in millisecond) */
	private static long max_RESET;
		/** The timeout const,delete(in millisecond) */
	private static long delete;
	private static long final_Delete;
		/** The timeout const,routing change(in millisecond) */
	private static long routing_CHANGE;
	
    private static FiniteStateMachineMetaOperations metaOps;
    protected static FiniteStateMachineMetaOperations registerMetaOperations ()
    {
        if (metaOps == null)
        {
            metaOps = new H2HMetaOperations();
        }
        return metaOps;
    }

	/**
	 * Constructs a MessageStoreFSM_H2HACK object with OL_Socket and MessageStore. 
	 * @param	Socket			OL_Socket
	 * @param   mstore			MessageStore
	 * @param	config			HyperCastConfig
	 */
	public MessageStoreFSM_H2HACK(OL_Socket Socket,MessageStore mstore,HyperCastConfig config){
		/*
		 * Initialization
		 */
		socket = Socket;
		MStore = mstore;
		MyState = Init;
		ACKReceived = new Vector();
		ACKExpected = new Vector();
		ACKSent = false;
		mylogicaladdress = socket.getLogicalAddress();

		/*
		 * Obtain the value of configurable parameter from the config object. It is only executed one time.
		 */
		 if (timeout_ACK < 0)
		 {
				timeout_ACK    = config.getPositiveLongAttribute 
						(XmlUtil.createXPath("/Public/NetworkService/EnhancedDelivery/LocalACK/TimeoutACK"));
				max_ACK        = config.getPositiveLongAttribute 
						(XmlUtil.createXPath("/Public/NetworkService/EnhancedDelivery/LocalACK/MaxTimeoutACK"));
				timeout_NACK   = config.getPositiveLongAttribute 
						(XmlUtil.createXPath("/Public/NetworkService/EnhancedDelivery/LocalACK/TimeoutNACK"));
				max_NACK       = config.getPositiveLongAttribute 
						(XmlUtil.createXPath("/Public/NetworkService/EnhancedDelivery/LocalACK/MaxTimeoutNACK"));
				timeout_RESET  = config.getPositiveLongAttribute 
						(XmlUtil.createXPath("/Public/NetworkService/EnhancedDelivery/LocalACK/TimeoutReset"));
				max_RESET      = config.getPositiveLongAttribute 
						(XmlUtil.createXPath("/Public/NetworkService/EnhancedDelivery/LocalACK/MaxTimeoutReset"));
				delete         = config.getPositiveLongAttribute 
						(XmlUtil.createXPath("/Public/NetworkService/EnhancedDelivery/LocalACK/Delete"));
				final_Delete   = config.getPositiveLongAttribute 
						(XmlUtil.createXPath("/Public/NetworkService/EnhancedDelivery/LocalACK/FinalDelete"));
				routing_CHANGE = config.getPositiveLongAttribute 
						(XmlUtil.createXPath("/Public/NetworkService/EnhancedDelivery/LocalACK/RoutingCheck"));
		 }

		/*
		 * In case that the finite state machine stuck at the Init state,
		 * it will delete itself when the final_Delete timeout occurs.
		 */
		
		MStore.setTimer(this,FINALDELETE_TIMER_INDEX,final_Delete);
	}
	/** 
	 * This method is used to adapt the dynamically changing tree
	 * structure of the overlay topology. It will take different actions
	 * depending on the current state of the finite state machine.
	 */	
	public void changingNeighbor()
	{
		 mylogicaladdress = socket.getLogicalAddress();
		 switch (MyState)
		 {
		 	case Init:
		 		break;
		 	case NoPayLoad:
		 	{
			 setNeighbor();
		 	}
			 break;
		 	case WaitforACK:
		 	{
		 		/*
		 		 * if leaf node, change to Done state
		 		 */
		 		if(length == 0)
		 		{
		 			MyState = Done;
		 			MStore.setTimer(this,DELETE_TIMER_INDEX,delete);
				}
		 		/*
		 		 * update the children nodes
		 		 */
		 		else 
		 		{
		 			if(deliveryMode == OL_Message.DELIVERY_MODE_MULTICAST)
		 			{
						I_LogicalAddress[] childrenArray = socket.getChildren(root);
						for(int i =0; i<childrenArray.length; i++){
							if(!ACKReceived.contains(childrenArray[i])&& !ACKExpected.contains(childrenArray[i]))
							{
								ACKExpected.addElement(childrenArray[i]);
							}
								
						}		
		 			}
		 			else if(deliveryMode == OL_Message.DELIVERY_MODE_UNICAST)
		 			{
						if(!ACKReceived.contains(socket.getParent(root))&& !ACKExpected.contains(socket.getParent(root)))
						{
							ACKExpected.addElement(socket.getParent(root));
						}
		 			}

		 		}
		 	}
			 break;
			 /*
			  * no action if in Done or Reset State
			  */
		 	case Done:
		 	case Reset:
			 break;
		 }
	}  
	
	/**
	 * @return the finite state machine id
	 */
	public short getFSMID()
	{
		return 	MessageStore.FSM_H2H_ACK;
	}
	

	/**Set the new ADF_Data message entry
	 * the information buffered includes the message id, the root of the tree
	 * and the payload data 
	 * @param	msg			OL_Message
	 */
	private void setADMsg(OL_Message msg)
	{
		if (message == null) message = msg;

		messageid = msg.getMessageIdentifier();
		deliveryMode = msg.getDeliveryMode();

		sourceaddress = msg.getSourceAddress();

		previousddress = msg.getPreviousHopAddress();

		switch (deliveryMode)
		{
			case OL_Message.DELIVERY_MODE_MULTICAST:
			{
				root = msg.getSourceAddress();
			}
				break;
			case OL_Message.DELIVERY_MODE_UNICAST:
			{
				root = msg.getDestinationAddress();	
			}
				break;

			default:
			throw new HyperCastWarningRuntimeException ("H2H ACK applies to unicast message and multicast message only");
					 
		}
		
		
	}
	
	/**Set the new ADF_Control message entry
	 * the information buffered includes the message id, the root of the tree
	 * @param	msg			OL_Message
	 */
	private void setACMsg(OL_Message msg)
	{
		messageid = msg.getMessageIdentifier();
		deliveryMode = ((FSM_Extension) msg.getFirstExtensionByType(Extension.FSM)).getDeliveryMode();


		/*
		 * each control message carries the root logical address
		 * this is necessary when the control message arrives earlier than the payload message
		 */

		root = socket.createLogicalAddress(((FSM_Extension) msg.getFirstExtensionByType(Extension.FSM)).getSenderAddress(),0);
	}
	
	/**
	 * Set the neighbor information
	 * This is calculated based on the root of the tree
	 */
	private void setNeighbor()
	{
		if (debug)
			System.out.println("MessageStoreFSM_H2HACK: setNeighbor: root="+root.toString()+", mylogicaladdress="+mylogicaladdress.toString());


		if(deliveryMode == OL_Message.DELIVERY_MODE_MULTICAST)
		{
			/*
			 * leaf node, no children node
			 */
			if(socket.getChildren(root)==null)
			{
				length = 0;
			}
			/*
			 * non-leaf node, update the children nodes
			 */
			else
			{
				I_LogicalAddress[] childrenArray = socket.getChildren(root);
				length = childrenArray.length;
				for(int i =0; i<childrenArray.length; i++)
				{
					ACKExpected.addElement(childrenArray[i]);
				}
			
			}
		}
		else if(deliveryMode == OL_Message.DELIVERY_MODE_UNICAST)
		{
			/*
			 * destination (or root) node
			 */
			 if(root.equals(mylogicaladdress))
			 {
			 	length = 0;
			 }
			 else
			 {
				length = 1;
				ACKExpected.addElement(socket.getParent(root));
			 }
		}

	}
	/**
	 * This is the method used to send ack or nack message to parent node.
	 * It can only be called when the root, message id are initiated.
	 * @param type  
	 */
	private void sendToParent(byte type, I_LogicalAddress destination)
	{


		OL_Message msg = (OL_Message) socket.createMessage(); //no payload
		FSM_Extension ee = new FSM_Extension(MessageStore.FSM_H2H_ACK, type, messageid, deliveryMode, root);
		msg.addExtension(ee);
				
		socket.sendToParent(msg, destination);
	}



	/**
	 * This is the method used to update the information about which node received the payload.
	 * It is created as a seperate method here to synchronize the add and remove operation.
	 */
	private synchronized void processAck(I_LogicalAddress la)
	{
			ACKReceived.addElement(la);
			if(ACKExpected.contains(la))
			{
				ACKExpected.removeElement(la);
			}
	}
	
	/**process a new formated application message
	 * @param	msg			OL_Message
	 */
	public void newADMsg(OL_Message msg)
	{
		/*
		 * Buffer the essential information such as message id, root of the tree
		 * and the payload 
		 */
		setADMsg(msg);

		/*
		 * Firstly, forward the message to the application
		 */
		if ((deliveryMode == OL_Message.DELIVERY_MODE_MULTICAST) ||
			((deliveryMode == OL_Message.DELIVERY_MODE_UNICAST) && (mylogicaladdress.equals(msg.getDestinationAddress()))))	
		{	
			if (debug)
				System.out.println("MessageStoreFSM_H2HACK: newADMsg: deliveryMode="+deliveryMode+", mylogicaladdress="+mylogicaladdress.toString()+",destination="+msg.getDestinationAddress().toString());

			socket.appmsgArrived(msg,socket.callback);
		}

		/*
		 * calculate the topology tree based on the root logical address
		 */
		setNeighbor();
		/*
		 * If all ack received, change the state to Done
		 */
		if(ACKExpected.isEmpty())
		{
			MyState = Done;
			MStore.setTimer(this,DELETE_TIMER_INDEX,delete);

		}
		/*
		 * If pending ack, change state to WaitforACK
		 */
		
		else{
			MyState = WaitforACK;
			MStore.setTimer(this,ACK_TIMER_INDEX,timeout_ACK);
			MStore.setTimer(this,MAXACK_TIMER_INDEX,max_ACK);
		}
		
		/*
		 * For non-source node, send the h2h ack to the parent node.
		 */
		if(!mylogicaladdress.equals(sourceaddress))
		{
			sendToParent(H2H_ACK,msg.getPreviousHopAddress());		
			ACKSent = true;
		}


	}
	
	/**Process the new ADF_Control message 
	 * @param	msg			OL_Message
	 */
	public void newACMsg(OL_Message msg)
	{
		/*
		 * store the root, the message id
		 * update the children nodes
		 */
		setACMsg(msg);
		setNeighbor();
		/*
		 * extract the format message header information
		 */
		FSM_Extension e = (FSM_Extension) msg.getFirstExtensionByType(Extension.FSM);
		/*
		 * state transit based on the message type
		 */
		switch(e.getType()){
			case H2H_ACK:
			case H2H_NACK:
			break;
			case H2H_REQUEST:
			{

				/*
				 * It is impossible to send a H2H_REQUEST control message to the source of the message.
				 * Below is safeguard.
				 */
				if (((deliveryMode == OL_Message.DELIVERY_MODE_MULTICAST) && (!mylogicaladdress.equals(root))) ||
					((deliveryMode == OL_Message.DELIVERY_MODE_UNICAST) &&
						((sourceaddress == null) || (!mylogicaladdress.equals(sourceaddress)))))
				{
					previousddress = msg.getSourceAddress();

					sendToParent(H2H_NACK, msg.getSourceAddress());

					/*
					 * change the state to NoPayLoad
					 */
					MyState = NoPayLoad;
					MStore.setTimer(this,NACK_TIMER_INDEX,timeout_NACK);
					MStore.setTimer(this,MAXNACK_TIMER_INDEX,max_NACK);
				}
				/*
				 * if the current node is the source, then some thing inconsistency happened,
				 * change the state to Reset.
				 */
				else 
				{
					MyState = Reset;
					MStore.setTimer(this,RESET_TIMER_INDEX,timeout_RESET);
					MStore.setTimer(this,MAXRESET_TIMER_INDEX,max_RESET);
				}
			}
				break;
			/*
			 * all other types of messages
			 * some thing inconsistency happened,change the state to Reset.
			 */	
			default:
			{
				MyState = Reset;
				MStore.setTimer(this,RESET_TIMER_INDEX,timeout_RESET);
				MStore.setTimer(this,MAXRESET_TIMER_INDEX,max_RESET);
			}
				break;
		}

	}	
	/**Update the known ADF_Data message entry
	 * @param	msg			OL_Message
	 */			
	public void updateADMsg(OL_Message msg)
	{
		if (message == null)
		{
			message = msg;
		}
		/*
		 * If the message is received before, don't process the comming packet. 
		 */
		else
		{
			return;
		}

		/*
		 * extract the format message header information
		 */
		FSM_Extension e = (FSM_Extension) msg.getFirstExtensionByType(Extension.FSM);

		/*
		 * If setADMsg has not yet been called, set the information of the message here.
		 */		
		if (sourceaddress == null)
		{
			messageid = msg.getMessageIdentifier();
			deliveryMode = msg.getDeliveryMode();
			sourceaddress = msg.getSourceAddress();
			previousddress = msg.getPreviousHopAddress();
		}
		
		/*
		 * take different actions at different state
		 */
		switch (MyState) {
			case Init:
			{
				/*
				 * forward the message to the application
				 */
				if ((deliveryMode == OL_Message.DELIVERY_MODE_MULTICAST) ||
					((deliveryMode == OL_Message.DELIVERY_MODE_UNICAST) && (mylogicaladdress.equals(msg.getDestinationAddress()))))	
				{	
					socket.appmsgArrived(msg,socket.callback);
				}
				/*
				 * if all ack received, change to Done state
				 */
				if(ACKExpected.isEmpty())
				{
					MyState = Done;
					MStore.setTimer(this,DELETE_TIMER_INDEX,delete);
				}
				/*
				 * pending ack, change to WaitforACK state
				 */
				else
				{
					MyState = WaitforACK;
					MStore.setTimer(this,ACK_TIMER_INDEX,timeout_ACK);
					MStore.setTimer(this,MAXACK_TIMER_INDEX,max_ACK);
				}
				/*
				 * send ack if not the root and has not sent the ack before
				 */
				if ((((deliveryMode == OL_Message.DELIVERY_MODE_MULTICAST) && (!mylogicaladdress.equals(root))) ||
					((deliveryMode == OL_Message.DELIVERY_MODE_UNICAST) &&
						((sourceaddress == null) || (!mylogicaladdress.equals(sourceaddress))))))
				{
					sendToParent(H2H_ACK, msg.getPreviousHopAddress());
					ACKSent = true;
				}
			}
				break;
			case NoPayLoad:	
			{
				/*
				 * forward the message to the application
				 */
				if ((deliveryMode == OL_Message.DELIVERY_MODE_MULTICAST) ||
					((deliveryMode == OL_Message.DELIVERY_MODE_UNICAST) && (mylogicaladdress.equals(msg.getDestinationAddress()))))	
				{	
					socket.appmsgArrived(msg,socket.callback);
				}
				
				/*
				 * not root, send ack
				 */
				if ((((deliveryMode == OL_Message.DELIVERY_MODE_MULTICAST) && (!mylogicaladdress.equals(root))) ||
					((deliveryMode == OL_Message.DELIVERY_MODE_UNICAST) &&
						((sourceaddress == null) || (!mylogicaladdress.equals(sourceaddress))))))
				{
					sendToParent(H2H_ACK,msg.getPreviousHopAddress());
					ACKSent = true;
				}
				/*
				 * all ack received, change to Done state
				 */
				if(ACKExpected.isEmpty())
				{ 
					MyState = Done;
					MStore.setTimer(this,DELETE_TIMER_INDEX,delete);				
				}
				/*
				 * pending ack, change to WaitforACK state
				 */
				else
				{
					/*
					 * forward the message to all its children nodes
					 */
					OL_Message retrans = new OL_Message (message);
					FSM_Extension ee = ((FSM_Extension) msg.getFirstExtensionByType(Extension.FSM));	
					ee.setType(PAYLOAD);
					
					if(deliveryMode == OL_Message.DELIVERY_MODE_MULTICAST)
					{
						retrans.setDeliveryMode(OL_Message.DELIVERY_MODE_MULTICAST);
						retrans.setDestinationAddress(null);
						retrans.setHopLimit((short) 255);						
						socket.forwardToChildren(retrans);
					}
					else if(deliveryMode == OL_Message.DELIVERY_MODE_UNICAST)
					{
						socket.forwardToParent(retrans);
					}						

					MyState = WaitforACK;
					MStore.setTimer(this,ACK_TIMER_INDEX,timeout_ACK);
					MStore.setTimer(this,MAXACK_TIMER_INDEX,max_ACK);

				}
			}
				break;
			/*
			 * no action if the current node has already obtained the payload
			 */	
			case(WaitforACK):
			case(Done):
				break;
		}
	}
	
	/**Update the new ADF_Control message 
	 * @param	msg			OL_Message
	 */
	public void updateACMsg(OL_Message msg)
	{
		/*
		 * extract the format message header information
		 */
		FSM_Extension e = (FSM_Extension) msg.getFirstExtensionByType(Extension.FSM);	
		/*
		 * take different actions at different state
		 */
		switch(MyState)
		{
		case Init:
			break;
		case NoPayLoad:
		{
			/*
			 * take actions based on message type
			 */
			switch(e.getType())
			{
				case H2H_REQUEST:
				{
					/*
					 * send nack
					 */
					if (((deliveryMode == OL_Message.DELIVERY_MODE_MULTICAST) && (!mylogicaladdress.equals(root))) ||
						((deliveryMode == OL_Message.DELIVERY_MODE_UNICAST) &&
							((sourceaddress == null) || (!mylogicaladdress.equals(sourceaddress)))))
					{
						sendToParent(H2H_NACK, msg.getSourceAddress());
					}
				}
					break;
				case H2H_NACK:
					break;
				case H2H_RESET:
				{
					MyState = Reset;
					MStore.setTimer(this,RESET_TIMER_INDEX,timeout_RESET);
					MStore.setTimer(this,MAXRESET_TIMER_INDEX,max_RESET);
				}
					break;
			}
		}
			break;
		case WaitforACK:
		{
			switch (e.getType()){
				case H2H_ACK:
				{
					/*
					 * update the nodes that have received the ack
					 */
					processAck(msg.getSourceAddress());	
					/*
					 * all ack received, change to Done state
					 */
					if(ACKExpected.isEmpty())
					{
						MyState = Done;
						MStore.setTimer(this,DELETE_TIMER_INDEX,delete);
						
					}
				}
					break;
				case H2H_REQUEST:
				{
					/*
					 * Even if the ACK has been sent by this node, the ACK may be lost. So whenever
					 * this node receives H2H_REQUEST, it needs to send a ACK.
					 */
					/*
					 * not root, has not sent ack
					 * send ack to parent node
					 */
					if ((((deliveryMode == OL_Message.DELIVERY_MODE_MULTICAST) && (!mylogicaladdress.equals(root))) ||
						((deliveryMode == OL_Message.DELIVERY_MODE_UNICAST) &&
							((sourceaddress == null) || (!mylogicaladdress.equals(sourceaddress))))))
					{
						sendToParent(H2H_ACK, msg.getSourceAddress());
						ACKSent = true;
					}
				}
					break;
					/*
					 * retransmit the payload mesage
					 */
				case H2H_NACK:
				{
					if (message == null)
					{
						return;
					}

					OL_Message retrans = new OL_Message (message);
					FSM_Extension ee = ((FSM_Extension) msg.getFirstExtensionByType(Extension.FSM));	
					ee.setType(PAYLOAD_RETRAN);
					
					if(deliveryMode == OL_Message.DELIVERY_MODE_MULTICAST)
					{
						retrans.setDeliveryMode(OL_Message.DELIVERY_MODE_UNICAST);
						retrans.setDestinationAddress(msg.getSourceAddress());
						retrans.setHopLimit((short) 1);						
						socket.forwardToParent(retrans);
					}
					else if(deliveryMode == OL_Message.DELIVERY_MODE_UNICAST)
					{
						socket.forwardToParent(retrans);
					}						

					MStore.incrementRetransmissionCount ();

				}
					break;
					/*
					 * change to Reset state
					 */
				case H2H_RESET:
				{
					MyState = Reset;
					MStore.setTimer(this,RESET_TIMER_INDEX,timeout_RESET);
					MStore.setTimer(this,MAXRESET_TIMER_INDEX,max_RESET);					
				}
			}
				
		}		
			break;
		case Done:
		{
			switch(e.getType()){
				case H2H_REQUEST:
				{
					/*
					 * Even if the ACK has been sent by this node, the ACK may be lost. So whenever
					 * this node receives H2H_REQUEST, it needs to send a ACK.
					 */
					/*
					 * not root, has not sent ack
					 * send ack to parent node
					 */
					if ((((deliveryMode == OL_Message.DELIVERY_MODE_MULTICAST) && (!mylogicaladdress.equals(root))) ||
						((deliveryMode == OL_Message.DELIVERY_MODE_UNICAST) &&
							((sourceaddress == null) || (!mylogicaladdress.equals(sourceaddress))))))
					{
						sendToParent(H2H_ACK, msg.getSourceAddress());
						ACKSent = true;
					}
				}
					break;
					/*
					 * retransmit the payload message
					 */
				case H2H_NACK:
				{
					if ((deliveryMode == OL_Message.DELIVERY_MODE_UNICAST) && (sourceaddress == null))
					{
						return;
					}

					OL_Message retrans = new OL_Message (message);
					FSM_Extension ee = ((FSM_Extension) msg.getFirstExtensionByType(Extension.FSM));	
					ee.setType(PAYLOAD_RETRAN);
					
					if(deliveryMode == OL_Message.DELIVERY_MODE_MULTICAST)
					{
						retrans.setDeliveryMode(OL_Message.DELIVERY_MODE_UNICAST);
						retrans.setDestinationAddress(msg.getSourceAddress());
						retrans.setHopLimit((short) 1);						
						socket.forwardToParent(retrans);
					}
					else if(deliveryMode == OL_Message.DELIVERY_MODE_UNICAST)
					{
						socket.forwardToParent(retrans);
					}						

					MStore.incrementRetransmissionCount ();

				}
					break;
					/*
					 * change state to Reset
					 */
				case H2H_RESET:
				{
					MyState = Reset;
					MStore.setTimer(this,RESET_TIMER_INDEX,timeout_RESET);
					MStore.setTimer(this,MAXRESET_TIMER_INDEX,max_RESET);
				}
					break;
			}
		}
			break;
		case Reset:	
			break;	
		
		}//end of switch
	}

		
	/**Set the time expired events
	 * @param	timerIndex		
	 */
	public void timerExpired(int timerIndex)
	{
		/*
		 * periodically check the topology changing
		 */
		 if(timerIndex==ROUTINGCHANGE_TIMER_INDEX)
		 {
			changingNeighbor();
			MStore.setTimer(this,ROUTINGCHANGE_TIMER_INDEX,routing_CHANGE);
		 }
		 /*
		  * take action based on state
		  */
		 switch(MyState){
			 case(Init):
			 {
				if(timerIndex==FINALDELETE_TIMER_INDEX)
				{
					MStore.remove(messageid);
				}
			 }
				 break;
			 case(NoPayLoad):
			 {
			 	/*
			 	 * Nack timeout, send nack
			 	 */
				if(timerIndex==NACK_TIMER_INDEX)
				{
				   if (deliveryMode == OL_Message.DELIVERY_MODE_MULTICAST)
				   {
					   if ((!root.equals(null)) && (!mylogicaladdress.equals(root)))
						   sendToParent(H2H_NACK,root);
				   }
				   else if (deliveryMode == OL_Message.DELIVERY_MODE_UNICAST)
				   {
					   if ((!previousddress.equals(null)) && (!mylogicaladdress.equals(previousddress)))
						   sendToParent(H2H_NACK,previousddress);
				   }
				}
				 /*
				  * max nack time out, change to Reset state
				  */
				 else if(timerIndex==MAXNACK_TIMER_INDEX)
				 {
					 MyState = Reset;
					 MStore.setTimer(this,RESET_TIMER_INDEX,timeout_RESET);
					 MStore.setTimer(this,MAXRESET_TIMER_INDEX,max_RESET);
				 }
			 }
				 break;
			 case(WaitforACK):
			 {
			 	/*
			 	 * ack timeout, send request to every children that has not acked back
			 	 */
				if(timerIndex==ACK_TIMER_INDEX )
				{
					OL_Message request = (OL_Message) socket.createMessage(); //no payload
					request.setDeliveryMode(OL_Message.DELIVERY_MODE_UNICAST);
		
					FSM_Extension ee = new FSM_Extension(MessageStore.FSM_H2H_ACK, H2H_REQUEST, messageid, deliveryMode, root);
					request.addExtension(ee);

					for (int i=0;  i<ACKExpected.size();i++)
					{
						I_LogicalAddress la = (I_LogicalAddress)ACKExpected.elementAt(i);
						request.setDestinationAddress(la);
						socket.forwardToParent(request);
					}
					MStore.setTimer(this,ACK_TIMER_INDEX,timeout_ACK);
				}
				/* 
				 * max ack time out, change to Done state
				 */
				else if((timerIndex==MAXACK_TIMER_INDEX) && (!ACKExpected.isEmpty()))
				{
					MyState = Done;
					MStore.setTimer(this,DELETE_TIMER_INDEX,delete);
				}
			 }
				 break;
			 case(Reset):
			 {
				if((timerIndex==RESET_TIMER_INDEX) && (length!=0))
				{

					if(deliveryMode == OL_Message.DELIVERY_MODE_MULTICAST)
					{
						I_LogicalAddress[] childaddress = socket.getChildren(root);
						for(int i=0;i<childaddress.length;i++)
						{
							sendToParent(H2H_RESET,childaddress[i]);
						}
					}
					else if(deliveryMode == OL_Message.DELIVERY_MODE_UNICAST)
					{
						if (!mylogicaladdress.equals(root)) //i.e., it is source.
						{
							sendToParent(H2H_RESET,root);
						}
					}

				}
				else if(timerIndex==MAXRESET_TIMER_INDEX)
				{
					MStore.remove(messageid);
				}
			 }
			
				 break;
			 case(Done):
			 {
				 if(timerIndex==DELETE_TIMER_INDEX){
					MyState = Init;
					/*
					 * delete itself
					 */
					MStore.remove(messageid);
				 }
			 }
				 break;
			}
	}
}

class H2HMetaOperations extends AcknowledgmentMetaOperations
{
	/*
	 * For an H2H unicast message, intermediate nodes will build FSM for it 
	 * in MessageStore.
	 */
	public boolean processIntermediateUnicastMessage(OL_Message message)
	{
		return true;
	}

    public NOTIFICATION_EVENT notifyOnSend (OL_Message message)
    {
        return null;
    }
}
