/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/
package hypercast.Pastry;

import java.io.Serializable;
import java.io.IOException;

import hypercast.I_PhysicalAddress;

/**
 * Provides an adapter between physical address objects used in
 * HyperCast and physical addresses used in the internal Pastry code.
 * The Pastry implementation couple the notion of being distributed
 * with the notino of having an Internet address. This coupling is
 * implemented in the "dist" directory of Pastry. HyperCast takes a
 * more general approach by using abstract physical address objects
 * rather than binding tightly to Internet addresses in particular. As
 * a result the code in the "dist" directory of Pastry could not be
 * used directly. In particular the HyperCast pastry driver
 * (rice/pastry/hc) is not an extension of the "dist" classes but
 * instead copies much of the "dist" code and replaces all occurrences
 * of Internet address classes (e.g. InetSocketAddress) with instances
 * of this class, HyperCastPhysicalAddress.
 *
 *  @author HyperCast Team
 *  @author Greg Mattes
 */
public final class HyperCastPhysicalAddress
    implements Serializable
{
    private I_PhysicalAddress pa;
    private String addressString;

    public HyperCastPhysicalAddress (final I_PhysicalAddress pa) {
        this.pa = pa;
    }

    public I_PhysicalAddress getPhysicalAddress (final HyperCastNodeAdapter hyperCastNodeAdapter) {
        return (pa != null) ? pa : (pa = hyperCastNodeAdapter.createPhysicalAddress (addressString));
    }

    public String toString() {
        return (addressString != null) ? addressString : (addressString = pa.toString());
    }

    private void writeObject (final java.io.ObjectOutputStream out)
        throws IOException
    {
        out.writeObject (toString());
    }

    private void readObject (final java.io.ObjectInputStream in)
        throws IOException, ClassNotFoundException
    {
        addressString = (String) in.readObject();
    }
}

