/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

package hypercast.Pastry;

import hypercast.*;

/**
 *  A physical, logical address pair for the Pastry protocol.
 *
 *  @author HyperCast Team
 *  @author Greg Mattes
 *  @version May 2004
 */
public final class Pastry_AddressPair
    implements I_AddressPair 
{
    private I_PhysicalAddress physicalAddress;
    private Pastry_LogicalAddress logicalAddress;

    Pastry_AddressPair (final I_PhysicalAddress physicalAddress, final Pastry_LogicalAddress logicalAddress) 
    {
        this.physicalAddress = physicalAddress;
        this.logicalAddress = logicalAddress;
    }

    public I_PhysicalAddress getPhysicalAddress() {
        return physicalAddress;
    }
    
    public I_LogicalAddress getLogicalAddress() {
        return logicalAddress;
    }

    /**
     *  Same as getLogicalAddress(), except the returned object type
     *  does not need to be coerced to type Pastry_LogicalAddress.
     */
    public Pastry_LogicalAddress getPastryLogicalAddress() {
        return logicalAddress;
    }

    public boolean equals (final Object object)
    {
        if (! (object instanceof Pastry_AddressPair))
            throw new IllegalArgumentException ("Object is not of type Pastry_AddressPair");

        Pastry_AddressPair other = (Pastry_AddressPair) object;

        boolean bothPAnull = (other.physicalAddress == null) && (this.physicalAddress == null);
        boolean bothLAnull = (other.logicalAddress == null) && (this.logicalAddress == null);
        
        boolean result =
            (bothPAnull || other.physicalAddress.equals (this.physicalAddress))
            &&
            (bothLAnull || other.logicalAddress.equals (this.logicalAddress));

        return result;
    }

    public String toString ()
    {
        return
            "physicalAddress:" + ((physicalAddress == null) ? null : physicalAddress.toString ()) +
            " " +
            "logicalAddress:" + ((logicalAddress == null) ? null : logicalAddress.toString ());
    }
}
