/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

package hypercast;

import java.util.*;

	/**
	* This class defines the OL_Message extension with type RAW_DATA. It carries
	* the application data. Only one this type of extension can be defined in
	* an overlay message.
	*
	* @author HyperCast Team
	*/

	public class PayloadExtension extends Extension {

		/** payload byte array  */
		protected byte[] Payload;
				
		
		/**
		 * Constructs an Extension header object with all the parameters
		 */
		public PayloadExtension(byte[] payload) {
			Payload = payload;
		}


		/**
		 * Constructs an Extension from an existing one
		 */
		public PayloadExtension(PayloadExtension e) {
			Payload = e.Payload;
		}
		
		
		/**
		 * Constructs an empty Extension
		 */
		public PayloadExtension() {
			Payload = null;
		}
		
		/**
		 * Return a new PayloadExtension extension instance by cloning this extension.
		 */
		public Extension Clone()
		{
			return new PayloadExtension(this);
		}

		/**
		 * Returns the type of extension.
		 */
		public byte getExtensionType() {
			return Extension.RAW_DATA;
		}


		/**
		 * Converts this object to byte array.
		 */
		public byte[] toByteArray() {
			return Payload;
		}


		/*
		 * This method provides a way for all extensions to get the length of 
		 * the byte array of an extension without actually executing toByteArray().
		 */
		/**
		 * Returns the length of the byte array of this extension.
		 */
		public int getSize()
		{
			return Payload.length;
		}


		/**
		 * Returns the payload
		 */
		public byte[] getPayload() {
 			return Payload;
		}

		
		/**
		 * Sets the payload 
		 */
		public void setPayload(byte[] payload) {
 			Payload = payload;
		}
		
            protected String payloadToString()
            {
                final String result = new String (Payload);
                return result;
            }
	}
