/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

package hypercast.SPT;

import hypercast.*;

/** A physical & logical address pair for the Spanning Tree Protocol. 
 *
 * @author HyperCast Team
 * @author Guangyu Dong
 * @version 0.1, May 25, 2003 
 */

public final class SPT_AddressPair implements I_AddressPair 
{
	/** the physical address */
    private I_PhysicalAddress PA;
    /** the logical address */
    private SPT_LogicalAddress LA;

	/**Constructor
	 * @param pa	physical address
	 * @param la	logical address
	 */
    public SPT_AddressPair(I_PhysicalAddress pa, SPT_LogicalAddress la) 
    {
        PA = pa;
        LA = la;
    }
    
	/** @return the physical address*/
	public I_PhysicalAddress getPhysicalAddress() 
    {
        return PA;
    }
    
	/** @return the logical address*/
	public I_LogicalAddress getLogicalAddress() 
    {
        return LA;
    }

	/** Same as getLogicalAddress(), except the returned object type does 
	 * not need to be coerced to type SPT_LogicalAddress. 
	 * @return the SPT logical address
	 */
	public SPT_LogicalAddress getSPTLogicalAddress() 
    {
        return LA;
    }

	/** @return "PA:" + PA.toString() + " LA:" + LA.toString()
	 */
	public String toString() 
    {
        return "PA:" + PA.toString() + " LA:" + LA.toString();
    }
    
    /** 
     * To check if it's equal to another address pair
     * @param ap	another address pair
     * @return		whether they are equal
     */
	public boolean equals(SPT_AddressPair ap)
	{
		return getPhysicalAddress().equals(ap.getPhysicalAddress()) 
				&& getLogicalAddress().equals(ap.getLogicalAddress());
	}
	public boolean equals(Object o){
		if(o==null)return false;
		if(!(o instanceof I_AddressPair))
			return false;
		I_AddressPair ap = (I_AddressPair)o;
		
		return LA.equals(ap.getLogicalAddress()) && PA.equals(ap.getPhysicalAddress());
		
	}
	/**Is it a better root/parent than ap?
	 * Used by SPT protocol to make parent choice
	 * @param ap	another address pair
	 * @return	whether this address pair better than the other address pair
	 */
	public boolean isBetterThan(SPT_AddressPair ap)
	{
		return 	getSPTLogicalAddress().isBetterThan(ap.getSPTLogicalAddress());
	}
	
}
