/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

package hypercast.SPT;

import hypercast.*;

import java.net.*;
import java.util.*;

/** 
 * A logical address on the Spanning Tree Network.
 * 
 * @author HyperCast Team
 * @author Guangyu Dong
 * @version	0.1, May 25, 2003
 */

public final class SPT_LogicalAddress extends SPT_ID implements I_LogicalAddress 
{
    /** length of byte array required to hold the address */
	private final static Random rand = new Random();
    
	public final static int ADDRESS_ARRAY_SIZE = 4;

	

	/** constructor with the integer value
	 * @param id	the ID in the integer form
	 */
	public SPT_LogicalAddress(int id) 
    {
        super(id);
    }
    /** Constructor
     * @param sptla		a spt logical address
     */
	public SPT_LogicalAddress(SPT_LogicalAddress sptla) 
    {
        super(sptla.getId());
    }

	/**Constructor
	 * Reads id from a byte array.
 	 * It reads 4 bytes 
	 * @param array		the byte array
	 * @param offset	the offset to read the array
	 */ 
 	public SPT_LogicalAddress(byte[] array, int offset) 
    {
        super(ByteArrayUtility.toInteger(array, offset));
    }



	/**Constructor
	 * @param s		the logical address in string form
	 */
	public SPT_LogicalAddress(String s) 
    {
        super(s);
    }
    
	/**
	 * Constructor
	 * Construct the address with ip address
	 * @param a		the ip address
	 */

	public SPT_LogicalAddress(INETv4OnePort a) 
    {
        //To run multiple nodes on a host, enlarge the range of x and y coordinate
        //super( everyOtherBit(a, true), everyOtherBit(a, false) );
        super(everyOtherBit(a, true)*50 + rand.nextInt() % 50+1); //prevent zero value
    }
	
	public void changeLogicalAddress(SPT_LogicalAddress newAddr){
		this.setId(newAddr.getId());
	}

	/** This function converts the IP_Address to a long, and
	 * then creates an integer made from every other bit of the IP_Address.
	 * It is called by <code>SPT_LogicalAddress(IP_Address)</code>
	 */

	private static int everyOtherBit(INETv4OnePort a, boolean evenBits) 
    {
        InetAddress ia = a.getInetAddress();
        byte[] b = ia.getAddress();

        // addressBites 0 to 7 are zero.
        //addressBits 8 to 39 contain the IP_Address
        // bits 40 to 55 are the port #.
        // bites 55 to 63 are random.
        int random8BitValue = Math.abs(rand.nextInt())%255;
        long addressBits = ((((((long) ByteArrayUtility.toInteger(b, 0)) & 0xFFFFFFFFL) << 16) + a.getPort()) << 8) +  random8BitValue;

        if (!evenBits)
        {
            addressBits >>>= 1;
        }

        int result = 0;
        for (int i = 0; i < 32; i++) 
        {
            result |= ((addressBits & 1) << i);
            addressBits >>>= 2;
        }

        return result;
    }



	/**
	 * @see hypercast.I_Address#greaterThan
	 */
	public boolean greaterThan(I_Address a) 
    {
        if (a == null || !(a instanceof SPT_LogicalAddress))
            throw new IllegalArgumentException("Attempted to compare SPT_LogicalAddress with something of a different type!");
        SPT_LogicalAddress dtla = (SPT_LogicalAddress) a;
        
        return SPT_ID.greaterThan(this, dtla);
    }
    /**
     * SPT node use this function to determine which node is better for root/parent
     * @param b		an address object
     * @return		if this address is better than the param address
     */
	public boolean isBetterThan(I_Address b){
		return !greaterThan(b) && !equals(b);
	}

		
	/** Returns the size of byte[] used to store address.
	 * @return 4
	 */
    
	public  int getSize() 
    {
        return ADDRESS_ARRAY_SIZE;
    }


	/** Converts the object into a byte array for transfer in a message.
	 * This is the exact inverse of the constructor that takes a byte array.
	 * @return the byte array
	 */
	public byte[] toByteArray() 
    {
        byte[] array = new byte[4];

        System.arraycopy(ByteArrayUtility.toByteArray(spt_id), 0, array, 0, 4);
        
        return array;
    }
}







