/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

package hypercast.util;
import hypercast.util.XmlUtil;

import java.io.File;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import java.io.InputStream;
import java.util.Properties;
import java.util.Vector;
import java.io.IOException;
import java.text.ParseException;
import java.io.FileNotFoundException;


/**
 * This class provides functions to create a Properties
 * object from a schema which uses all possible Xpath 
 * string as entry and its default values as value. 
 *
 * @author HyperCast Team
 * @version 2005 (version 3.0)
 */
public class SchemaDefaultValueCalculator
{
	/**
	 * debug flag
	 */
	final static boolean debug = false;
	
	/** The schema file. */
	private File SchemaFile = null;

	/** The schema file name. */
	private String schemaFileName = null;

	/** The Document instance created from the opened schema file. */
	private Document openedSchemaFile = null;


    /**
     * Constructor.
     */
    public SchemaDefaultValueCalculator(String schemafile)
    throws IOException, ParseException
	{
		schemaFileName = schemafile;

		SchemaFile = new File(schemafile);

		if (schemaFileName.toLowerCase().endsWith(".xsd"))
		{
			/*
			 * Open schema file.
			 */
			try
			{ 
				openedSchemaFile = XmlUtil.createDocument(schemafile);
			}
			catch (final IOException ioe)
			{
				throw new IOException("Cannot open schema file "+schemaFileName+": "+ioe);
			}
			catch (final ParseException ioe)
			{
				throw new ParseException("Failed in parsingschema  file "+schemaFileName+": "+ioe, 0);
			}

			if (openedSchemaFile == null)
			{
				throw new IOException("Poor formated schema file: hypercast.xsd");
			}
		}
		else
		{		
		}
    }

	/**
	 * Constructor.
	 */
	public SchemaDefaultValueCalculator()
	throws IOException
	{
		schemaFileName = "hypercast.xsd";

		InputStream fis = getClass().getResourceAsStream( schemaFileName );
				
		if (fis == null)
		{
			System.out.println("Connot find default schema file "+schemaFileName+".");
			return;
		}
				
		/*
		 * Open schema file.
		 */
		try
		{ 
			openedSchemaFile = XmlUtil.createDocument(fis, new String("hypercast.xsd"));
		}
		catch (final IOException ioe)
		{
			throw new IOException("Cannot open schema file "+schemaFileName+": "+ioe);
		}
				
		if (openedSchemaFile == null)
		{
			throw new IOException("Poor formated schema file: hypercast.xsd");
		}
	}

	/**
	 * Build the Properties object for all default values of a schema file.
	 */
	public Properties getDefaultValues()
		throws IOException
	{
		if (openedSchemaFile == null)
		{
			schemaFileName = "hypercast.xsd";

			InputStream fis = getClass().getResourceAsStream( schemaFileName );
				
			if (fis == null)
			{
				System.out.println("Connot find default schema file "+schemaFileName+".");
				return new Properties();
			}
				
			/*
			 * Open schema file.
			 */
			try
			{ 
				openedSchemaFile = XmlUtil.createDocument(fis, new String("hypercast.xsd"));
			}
			catch (final IOException ioe)
			{
				throw new IOException("Cannot open schema file "+schemaFileName+": "+ioe);
			}
				
			if (openedSchemaFile == null)
			{
				throw new IOException("Poor formated schema file: hypercast.xsd");
			}
		}

		return getDefaultValues(openedSchemaFile);
	}

	/**
	 * Build the Properties object for all default values of a schema file.
	 */
	public static Properties getDefaultValues(String schemafile)
		throws FileNotFoundException, ParseException, IOException
	{
		Document schemaDoc = null;

		if (schemafile.toLowerCase().endsWith(".xsd"))
		{
			/*
			 * Open schema file.
			 */
			schemaDoc = XmlUtil.createDocument(schemafile);

			if (schemaDoc == null)
			{
				throw new IOException("Poor formated schema file: "+schemafile);
			}
		}
		else
		{		
			System.err.println("The file "+schemafile+" is not a schema file.");
		}
		
		return getDefaultValues(schemaDoc);
	}
	
    /**
     * Build the Properties object for all default values of a schema file.
     */
    public static Properties getDefaultValues(Document schemaDoc)
    {
		Properties defaultProperties = new Properties();

		if (schemaDoc == null)
		{
			return defaultProperties;
		}
		
		/*
		 * Find the property Element in schema file. The way to find
		 * property Element is hypercast specific.
		 */
		Element propertyElement = HypercastConfigurator.findPropertyElement(schemaDoc);

		if (propertyElement == null)
		{
			System.err.println("Poor formated schema file: " + schemaDoc.getNodeName());
			return null;						
		}

		if (debug)
			System.out.println("find property Element in the given Document: "+schemaDoc.getNodeName());

		recursiveGetDefaultValues("", defaultProperties, propertyElement);

		return defaultProperties;
    }
    
	/**
	 * Recursively get all default values in the schema file and put them in 
	 * the given Properties instance.
	 * 
	 * @param prefix	the string which represents the xpath string to current Element or Node.
	 * @param p	a Properties instabce in which the default values are stored.
	 * @param xmlElement an Element in the schema DOM which represents an attribute.
	 */
	private static void recursiveGetDefaultValues(String prefix, Properties p, Element xmlElement)
	{
		if (debug)
			System.out.println("Process Element: "+xmlElement.getAttribute("name"));

		/*
		 * Handle complexType Element.
		 */
		if (HypercastConfigurator.isComplexElement(xmlElement))
		{
			processComplexElement(prefix, p, xmlElement);
		}
		/*
		 * Otherwise it is simpleType Element
		 */
		else
		{
			processSimpleElement(prefix, p, xmlElement);
		}
	}


	/**
	 * Get the default value of the given complexType Element e and recursively process its children.
	 * 
	 * @param prefix	the string which represents the xpath string to current Element or Node.
	 * @param p	a Properties instabce in which the default values are stored.
	 * @param e an Element in the schema DOM which represents attribute.
	 */
	private static void processComplexElement(String prefix, Properties p, Element e)
	{
		String name = e.getAttribute("name");

		if (debug)
			System.out.println("Process complexType Element: "+name);
			
		String newPrefix = new String(prefix+"/"+name);

		/*
		 * Get all children of this Element in the schema file.
		 */
		Vector children = HypercastConfigurator.getChildrenOfComplexElement(e);
		
		/*
		 * Only when given compleType Element has choice on its children, the default
		 * value is created for it.
		 */
		if (HypercastConfigurator.hasChoiceOnChildren(e))
		{
			String defaultValue = e.getAttribute("default");

			if ((defaultValue == "") || (defaultValue == null))
			{
				Element firstChild = (Element) children.elementAt(0);//the first child name because default value.
				
				if (firstChild != null)
				{
					String childName = firstChild.getAttribute("name");

					if (debug)
						System.out.println("Entry is added to Properties: "+newPrefix);

					p.put(new String(newPrefix), childName);
				}
				else
				{
					System.out.println("The complexType Element "+name+" doesn't have child.");
				}

			}
			else
			{
				if (debug)
					System.out.println("Entry is added to Properties: "+newPrefix);

				p.put(new String(newPrefix), defaultValue);
			}
		}

		/*
		 * This loop is to process this node's children recursively.
		 */
		for (int i=0; i<children.size(); i++)
		{
			Element child = (Element) children.elementAt(i);
				
			if (child == null)
			{
				continue;
			}

			if (debug)
				System.out.println("Loop on the children of "+newPrefix);


			recursiveGetDefaultValues(newPrefix, p, child);
		}

	}

	/**
	 * Get the default of given simpleType Element.
	 * 
	 * @param prefix	the string which represents the xpath string to current Element or Node.
	 * @param p	a Properties instabce in which the default values are stored.
	 * @param e an Element in the schema DOM which represents attribute.
	 */
	private static void processSimpleElement(String prefix, Properties p, Element e)
	{
		String name = e.getAttribute("name");

		if (debug)
			System.out.println("Process simpleType Element: "+name);

		if (HypercastConfigurator.getRestrictionElement(e) == null)
		{
			return;
		}
		
		String newPrefix = new String(prefix+"/"+name);

		String defaultValue = e.getAttribute("default");

		if (defaultValue != null)
		{
			if (debug)
				System.out.println("Loop on the children of "+newPrefix);

			p.put(new String(newPrefix), defaultValue);
		}
	}
    
}
