/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/
package hypercast.util;

import java.io.File;
import java.util.Hashtable;
import java.util.Enumeration;
import javax.swing.filechooser.*;

/**
 * This class implements a file filter in which file extensions are 
 * defined. It can be used by a file chooser to open a file with
 * specified type or extension. 
 *
 * @author HyperCast Team
 * @version 2005 (version 3.0)
 */
public class TypeBased_FileFilter extends FileFilter
{

    private Hashtable filters = null;
    private String description = null;
    private String fileTypeList = null;

    /**
     * Constructor. If It creates a file filter that accepts
     * all files (no filter).
     */
    public TypeBased_FileFilter()
	{
		this.filters = new Hashtable();
    }

    /**
     * Check if the given file should be shown in the file chooser pane.
     * Return true if it is defined in this file filter; false otherwise.
     */
    public boolean accept(File file) {
		if (file == null)
		{
			return false;
		}
		else
		{
		    if (file.isDirectory())
		    {
				return true;
		    }
		    
		    String fileType = getFileType(file);
		    if((fileType != null) && (filters.get(fileType) != null))
		    {
				return true;
		    }
		    else
		    {
				return false;
		    }
		}
    }

	/**
	 * Gets the description string on the file types of this filter.
	 */
	public String getDescription()
	{
		if(fileTypeList == null)
		{
			fileTypeList = (description==null ? "(" : description + "(");
			Enumeration extensions = filters.keys();
			if(extensions != null) {
				fileTypeList += "." + (String) extensions.nextElement();
				while (extensions.hasMoreElements())
				{
					fileTypeList += ", ." + (String) extensions.nextElement();
				}
			}
			fileTypeList += ")";
		}
		return fileTypeList;
	}

    /**
     * Return the type or extension portion of the file's name .
     */
    public String getFileType(File file)
    {
		if(file != null)
		{
		    String filename = file.getName();
		    int index = filename.lastIndexOf('.');
		    if(index>0 && index<filename.length()-1)
		    {
				return filename.substring(index+1).toLowerCase();
		    };
		}
		return null;
    }

    /**
     * Adds a file type (or file extension) into this filter.
     */
    public void addFileType(String fileType)
    {
		filters.put(fileType.toLowerCase(), this);
		fileTypeList = null;
    }

    /**
     * Sets the description on the file types that is displayed in the file
     * chooser pane to help user to choose files.
     */
    public void setDescription(String description)
    {
		this.description = description;
		fileTypeList = null;
    }

}
